'use strict';

var CL = require('./coastline');
var fs = require('fs');

var success = Symbol('success');

var assert = function (cond) {
	if (!cond) CL.throw('CLTAssert', 'Test assertion failed');
};

var eq = function (a, b) {
	if (a !== b && JSON.stringify(a) !== JSON.stringify(b)) {
		console.log(JSON.stringify(a));
		console.log(JSON.stringify(b));
		CL.throw('CLTEq', 'Test assertion failed');
	}
};

module.exports = {

	CL: CL,
	
	assert: assert,
	eq: eq,
	success: success
	
};

CL.run(function* () {

	var failed = [];
	
	var files;
	if (process.argv[2]) files = [process.argv[2]];
	else files = yield fs.readdir(__dirname + '/tests', CL.cb());
	
	for (var file of files) {
		
		var res = null;
		yield CL.try(function *() {
			res = yield require(__dirname + '/tests/' + file);
		}, function* (e) {
			console.log(e.stack);
		});
		
		if (res === success) {
			console.log(file, 'success');
		}
		else {
			console.log(file, 'FAIL');
			failed.push(file);
		}
		
	}
	
	if (failed.length) {
		console.log(failed.length, 'tests failed:', failed);
	}
	else {
		console.log('All tests succeeded');
	}
	
});