'use strict';

var CLT = require('../test');
var CL = CLT.CL;
var fs = require('fs');

module.exports = CL.run(function* () {

	var log = [];
	
	yield CL.try(function* () {
		
		log.push('try-1');
		
		yield CL.try(function* () {
			log.push('try-enoent');
			yield fs.readFile('does.not.exist', CL.cb());
			log.push('wtf-enoent');
		}, {
			ENOENT: function* () {
				log.push('catch-enoent');
			}
		});
		
		yield CL.try(function* () {
			log.push('try-custom');
			yield fs.readFile('does.not.exist', CL.cb('custom'));
			log.push('wtf-custom');
		}, {
			custom: function* () {
				log.push('catch-custom');
			}
		});
		
		yield CL.try(function* () {
			log.push('try-str');
			CL.throw('type-str', 'msg-str');
			log.push('wtf-str');
		}, {
			'*': function* (e) {
				log.push(e.type);
				log.push(e.message);
			}
		});
		
		yield CL.try(function* () {
			log.push('try-obj');
			CL.throw('type-obj', { str: 'data-obj' });
			log.push('wtf-obj');
		}, {
			'*': function* (e) {
				log.push(e.type);
				log.push(e.data.str);
			}
		});
		
		yield CL.try(function* () {
			log.push('try-sstr');
			CL.throw('sstr');
			log.push('wtf-sstr');
		}, {
			'*': function* (e) {
				log.push(e.type);
				log.push(e.message);
			}
		});
		
		yield CL.try(function* () {
			log.push('try-sobj');
			CL.throw({ str: 'sobj' });
			log.push('wtf-sobj');
		}, {
			'*': function* (e) {
				log.push(e.type);
				log.push(e.data.str);
			}
		});
		
		yield CL.try(function* () {
			log.push('try-pysh');
			log.pysh('wtf-pysh-1');
			log.push('wtf-pysh-2');
		}, {
			NotTypeError: function* () {
				log.push('wtf-pysh-3');
			}
		});
		
		log.push('wtf-1');
		
	}, function (e) {
		log.push('catch-1');
	});
	
	yield CL.try(function* () {
		log.push('try-2');
		
		yield CL.try(function* () {
			log.push('try-pysh');
			log.pysh('wtf-pysh-1');
			log.push('wtf-pysh-2');
		}, {
			'*': function* () {
				log.push('catch-pysh');
				log.pysh('wtf-pysh-3');
				log.push('wtf-pysh-4');
			}
		});
		
		log.push('wtf-2');
	}, function (e) {
		log.push('catch-2');
	});
	
	yield CL.try(function* () {
		log.push('try-3');
		
		yield CL.run(function () {
			log.push('try-meth');
			log.pysh('wtf-meth-1');
		});
		
		log.push('wtf-meth-2');
	}, function (e) {
		log.push('catch-3');
	});
	
	yield CL.try(function* () {
		log.push('try-4');
		
		var bg = CL.bg(function* () {
			log.push('try-bg');
			log.pysh('wtf-bg-1');
		});
		
		yield CL.sleep(1);
		log.push('cont-bg');
		
		yield bg;
		
		log.push('wtf-bg-2');
	}, function (e) {
		log.push('catch-4');
	});
	
	yield CL.try(function* () {
		log.push('try-5');
		
		var bg = CL.bg(function* () {
			log.push('try-bgw');
			yield CL.sleep(100);
			CL.throw('asdf');
		});
		
		yield CL.sleep(1);
		log.push('cont-bgw');
		
		yield bg;
		
		log.push('wtf-bgw-2');
	}, function (e) {
		log.push('catch-5');
	});
	
	var host = {
		err: CL.fun(function* () {
			log.push('try-host');
			yield this.err2();
		}),
		err2: CL.fun(function* () {
			CL.throw('err-host');
		})
	};
	yield CL.try(function* () {
		log.push('try-6');
		yield host.err();
	}, function* (e) {
		log.push(e.type);
		log.push('catch-6');
	});
	yield CL.try(function* () {
		log.push('try-7');
		yield host.err();
	}, function* (e) {
		log.push(e.type);
		log.push('catch-7');
	});
	
	CLT.eq(log, [
		'try-1',
		'try-enoent', 'catch-enoent',
		'try-custom', 'catch-custom',
		'try-str', 'type-str', 'msg-str',
		'try-obj', 'type-obj', 'data-obj',
		'try-sstr', 'sstr', 'sstr',
		'try-sobj', 'Error', 'sobj',
		'try-pysh',
		'catch-1',
		'try-2', 'try-pysh', 'catch-pysh', 'catch-2',
		'try-3', 'try-meth', 'catch-3',
		'try-4', 'try-bg', 'cont-bg', 'catch-4',
		'try-5', 'try-bgw', 'cont-bgw', 'catch-5',
		'try-6', 'try-host', 'err-host', 'catch-6',
		'try-7', 'try-host', 'err-host', 'catch-7'
	]);
	
	var ret;
	
	ret = yield CL.try(function* () {
		return 'success';
	}, function* () {
		return 'fail';
	});
	CLT.eq(ret, 'success');
	
	ret = yield CL.try(function* () {
		CL.throw('fail');
		return 'success';
	}, function* () {
		return 'fail';
	});
	CLT.eq(ret, 'fail');
	
	ret = yield CL.try(function* () {
		yield (ecb => ecb())(CL.error());
		return 'success';
	}, function* () {
		return 'fail';
	});
	CLT.eq(ret, 'fail');
	
	return CLT.success;
	
});
