'use strict';

var CLT = require('../test');
var CL = CLT.CL;

module.exports = CL.run(function* () {
	
	var count;
	var sum;
	var t0;
	var arr = [0, 1, 2, 3, 4];
	var failed;
	
	// test with limit > tasks
	count = sum = 0;
	t0 = Date.now();
	yield CL.parallel(arr, function* (v, i) {
		yield CL.sleep(v * 100);
		CLT.eq(v, i);
		count++;
		sum += v;
	});
	
	CLT.eq(count, 5);
	CLT.eq(sum, 10);
	CLT.eq(Math.round((Date.now() - t0) / 100), 4);
	
	// test with limit < tasks
	count = sum = 0;
	t0 = Date.now();
	yield CL.parallel(arr, function* (v, i) {
		yield CL.sleep(v * 100);
		CLT.eq(v, i);
		count++;
		sum += v;
	}, 1);
	
	CLT.eq(count, 5);
	CLT.eq(sum, 10);
	CLT.eq(Math.round((Date.now() - t0) / 100), 10);
	
	var obj = {
		five: 5,
		four: 4,
		three: 3,
		two: 2,
		one: 1
	};
	var out = [];
	t0 = Date.now();
	yield CL.parallel(obj, CL.fun(function* (v, k) {
		yield CL.sleep(v * 100);
		out.push(k);
	}));
	
	CLT.eq(Math.round((Date.now() - t0) / 100), 5);
	CLT.eq(out, ['one', 'two', 'three', 'four', 'five']);
	
	// error with limit < tasks
	failed = 0;
	yield CL.try(function* () {
		yield CL.parallel(obj, function* (v, k) {
			CL.throw('fail');
		}, 2);
	}, {
		fail: function* () {
			failed++;
		}
	});
	CLT.eq(failed, 1);
	
	// error with limit > tasks
	failed = 0;
	yield CL.try(function* () {
		yield CL.parallel(obj, function* (v, k) {
			CL.throw('fail');
		});
	}, {
		fail: function* () {
			failed++;
		}
	});
	CLT.eq(failed, 1);
	
	// error, then success
	failed = 0;
	yield CL.try(function* () {
		yield CL.parallel([true, false], function* (v, k) {
			if (v) {
				yield CL.sleep(100);
				CL.throw('fail');
			}
			else {
				yield CL.sleep(200);
			}
		}, 2);
	}, {
		fail: function* () {
			failed++;
		}
	});
	CLT.eq(failed, 1);
	
	return CLT.success;
	
});
