'use strict';

var CLT = require('../test');
var CL = CLT.CL;
var fs = require('fs');

module.exports = CL.run(function* () {

	var res;
	
	res = yield new Promise(function (resolve, reject) {
		resolve('ok');
	});
	CLT.eq(res, 'ok');

	res = yield new Promise(function (resolve, reject) {
		setTimeout(function () {
			resolve('ok')
		}, 100);
	});
	CLT.eq(res, 'ok');
	
	yield CL.try(function* () {
		yield new Promise(function (resolve, reject) {
			reject('fail');
		});
	}, function* (e) {
		res = e.type;
	});
	CLT.eq(res, 'fail');
	
	res = yield new Promise(function (resolve, reject) {
		CL.top(function* () {
			CL.sleep(100);
			return 'ok';
		}).then(resolve, reject);
	});
	CLT.eq(res, 'ok');
	
	new Promise(function (resolve, reject) {
		CL.top(function* () {
			CL.throw('fail');
		}).then(resolve, reject);
	}).then(null, function (e) {
		res = e.type;
	});
	yield CL.sleep(100);
	CLT.eq(res, 'fail');
	
	var ready = CL.run(function* () {
		return 'ok';
	});
	yield ready;
	res = yield new Promise(function (resolve, reject) {
		ready.then(resolve, reject);
	});
	CLT.eq(res, 'ok');
	
	var err;
	yield CL.try(function* () {
		err = CL.run(function* () {
			CL.throw('fail');
		});
		yield err;
	}, function* (e) {
		// noop
	});
	yield new Promise(function (resolve, reject) {
		err.then(resolve, reject);
	}).then(null, function (e) {
		res = e.type;
	});
	CLT.eq(res, 'fail');
	
	var bga = CL.bg(new Promise(function (resolve, reject) {
		resolve('ok-1');
	}));
	var bgb = CL.bg(new Promise(function (resolve, reject) {
		setTimeout(function () {
			resolve('ok-2')
		}, 100);
	}));
	CLT.eq(yield CL.wait([bga, bgb]), ['ok-1', 'ok-2']);
	
	res = yield new Promise(function (resolve, reject) {
		resolve('ok');
	}).then(CL.done(), CL.error());
	CLT.eq(res, 'ok');
	
	res = yield new Promise(function (resolve, reject) {
		setTimeout(function () {
			resolve('ok')
		}, 100);
	}).then(CL.done(), CL.error());
	CLT.eq(res, 'ok');
	
	yield CL.try(function* () {
		yield new Promise(function (resolve, reject) {
			reject('fail');
		}).then(CL.done(), CL.error());
	}, function* (e) {
		res = e.type;
	});
	CLT.eq(res, 'fail');
	
	res = 'nope';
	yield new Promise(function (resolve, reject) {
		err.then(resolve, null).then(null, reject);
	}).then(null, function (e) {
		res = e.type;
	});
	CLT.eq(res, 'fail');
	
	res = yield new Promise(function (resolve, reject) {
		CL.top(function* () {
			yield CL.sleep(100);
			return 'o';
		}).then(function (o) {
			return o + 'k';
		}).then(function (ok) {
			return new Promise(function (resolve, reject) {
				setTimeout(function () {
					resolve(ok + '!');
				}, 100);
			});
		}).then(resolve);
	});
	CLT.eq(res, 'ok!');
	
	res = yield new Promise(function (resolve, reject) {
		CL.top(function* () {
			yield CL.sleep(100);
			return 'o';
		}).then(function (o) {
			throw new Error(o + 'k');
		}).then(null, function (ok) {
			return new Promise(function (resolve, reject) {
				setTimeout(function () {
					resolve(ok.message + '!');
				}, 100);
			});
		}).then(resolve);
	});
	CLT.eq(res, 'ok!');
	
	return CLT.success;
	
});